@@ -10,5 +10,5 @@ from PIL import Image
 def test_crashes(test_file):
     with open(test_file, "rb") as f:
         im = Image.open(f)
-        with pytest.raises(IOError):
+        with pytest.raises(OSError):
             im.load()
