@@ -26,7 +26,6 @@
 #include "Python.h"
 
 #include "Imaging.h"
-#include "py3.h"
 #include "Gif.h"
 
 #ifdef HAVE_UNISTD_H
@@ -56,12 +55,14 @@ PyImaging_EncoderNew(int contextsize)
     ImagingEncoderObject *encoder;
     void *context;
 
-    if(PyType_Ready(&ImagingEncoderType) < 0)
+    if(PyType_Ready(&ImagingEncoderType) < 0) {
         return NULL;
+    }
 
     encoder = PyObject_New(ImagingEncoderObject, &ImagingEncoderType);
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
     /* Clear the encoder state */
     memset(&encoder->state, 0, sizeof(encoder->state));
@@ -74,8 +75,9 @@ PyImaging_EncoderNew(int contextsize)
             (void) PyErr_NoMemory();
             return NULL;
         }
-    } else
+    } else {
         context = 0;
+    }
 
     /* Initialize encoder context */
     encoder->state.context = context;
@@ -94,8 +96,9 @@ PyImaging_EncoderNew(int contextsize)
 static void
 _dealloc(ImagingEncoderObject* encoder)
 {
-    if (encoder->cleanup)
+    if (encoder->cleanup) {
         encoder->cleanup(&encoder->state);
+    }
     free(encoder->state.buffer);
     free(encoder->state.context);
     Py_XDECREF(encoder->lock);
@@ -126,19 +129,22 @@ _encode(ImagingEncoderObject* encoder, PyObject* args)
 
     Py_ssize_t bufsize = 16384;
 
-    if (!PyArg_ParseTuple(args, "|n", &bufsize))
+    if (!PyArg_ParseTuple(args, "|n", &bufsize)) {
         return NULL;
+    }
 
     buf = PyBytes_FromStringAndSize(NULL, bufsize);
-    if (!buf)
+    if (!buf) {
         return NULL;
+    }
 
     status = encoder->encode(encoder->im, &encoder->state,
                              (UINT8*) PyBytes_AsString(buf), bufsize);
 
     /* adjust string length to avoid slicing in encoder */
-    if (_PyBytes_Resize(&buf, (status > 0) ? status : 0) < 0)
+    if (_PyBytes_Resize(&buf, (status > 0) ? status : 0) < 0) {
         return NULL;
+    }
 
     result = Py_BuildValue("iiO", status, encoder->state.errcode, buf);
 
@@ -180,14 +186,16 @@ _encode_to_file(ImagingEncoderObject* encoder, PyObject* args)
     Py_ssize_t fh;
     Py_ssize_t bufsize = 16384;
 
-    if (!PyArg_ParseTuple(args, "n|n", &fh, &bufsize))
+    if (!PyArg_ParseTuple(args, "n|n", &fh, &bufsize)) {
         return NULL;
+    }
 
     /* Allocate an encoder buffer */
     /* malloc check ok, either constant int, or checked by PyArg_ParseTuple */
     buf = (UINT8*) malloc(bufsize);
-    if (!buf)
+    if (!buf) {
         return PyErr_NoMemory();
+    }
 
     ImagingSectionEnter(&cookie);
 
@@ -198,12 +206,13 @@ _encode_to_file(ImagingEncoderObject* encoder, PyObject* args)
 
         status = encoder->encode(encoder->im, &encoder->state, buf, bufsize);
 
-        if (status > 0)
+        if (status > 0) {
             if (write(fh, buf, status) < 0) {
                 ImagingSectionLeave(&cookie);
                 free(buf);
-                return PyErr_SetFromErrno(PyExc_IOError);
+                return PyErr_SetFromErrno(PyExc_OSError);
             }
+        }
 
     } while (encoder->state.errcode == 0);
 
@@ -229,11 +238,13 @@ _setimage(ImagingEncoderObject* encoder, PyObject* args)
     x0 = y0 = x1 = y1 = 0;
 
     /* FIXME: should publish the ImagingType descriptor */
-    if (!PyArg_ParseTuple(args, "O|(nnnn)", &op, &x0, &y0, &x1, &y1))
+    if (!PyArg_ParseTuple(args, "O|(nnnn)", &op, &x0, &y0, &x1, &y1)) {
         return NULL;
+    }
     im = PyImaging_AsImaging(op);
-    if (!im)
+    if (!im) {
         return NULL;
+    }
 
     encoder->im = im;
 
@@ -265,8 +276,9 @@ _setimage(ImagingEncoderObject* encoder, PyObject* args)
         state->bytes = (state->bits * state->xsize+7)/8;
         /* malloc check ok, overflow checked above */
         state->buffer = (UINT8*) malloc(state->bytes);
-        if (!state->buffer)
+        if (!state->buffer) {
             return PyErr_NoMemory();
+        }
     }
 
     /* Keep a reference to the image object, to make sure it doesn't
@@ -285,8 +297,9 @@ _setfd(ImagingEncoderObject* encoder, PyObject* args)
     PyObject* fd;
     ImagingCodecState state;
 
-    if (!PyArg_ParseTuple(args, "O", &fd))
+    if (!PyArg_ParseTuple(args, "O", &fd)) {
         return NULL;
+    }
 
     state = &encoder->state;
 
@@ -387,8 +400,9 @@ PyImaging_EpsEncoderNew(PyObject* self, PyObject* args)
     ImagingEncoderObject* encoder;
 
     encoder = PyImaging_EncoderNew(0);
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
     encoder->encode = ImagingEpsEncode;
 
@@ -409,15 +423,18 @@ PyImaging_GifEncoderNew(PyObject* self, PyObject* args)
     char *rawmode;
     Py_ssize_t bits = 8;
     Py_ssize_t interlace = 0;
-    if (!PyArg_ParseTuple(args, "ss|nn", &mode, &rawmode, &bits, &interlace))
+    if (!PyArg_ParseTuple(args, "ss|nn", &mode, &rawmode, &bits, &interlace)) {
         return NULL;
+    }
 
     encoder = PyImaging_EncoderNew(sizeof(GIFENCODERSTATE));
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
-    if (get_packer(encoder, mode, rawmode) < 0)
+    if (get_packer(encoder, mode, rawmode) < 0) {
         return NULL;
+    }
 
     encoder->encode = ImagingGifEncode;
 
@@ -474,15 +491,18 @@ PyImaging_RawEncoderNew(PyObject* self, PyObject* args)
     Py_ssize_t stride = 0;
     Py_ssize_t ystep = 1;
 
-    if (!PyArg_ParseTuple(args, "ss|nn", &mode, &rawmode, &stride, &ystep))
+    if (!PyArg_ParseTuple(args, "ss|nn", &mode, &rawmode, &stride, &ystep)) {
         return NULL;
+    }
 
     encoder = PyImaging_EncoderNew(0);
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
-    if (get_packer(encoder, mode, rawmode) < 0)
+    if (get_packer(encoder, mode, rawmode) < 0) {
         return NULL;
+    }
 
     encoder->encode = ImagingRawEncode;
 
@@ -506,15 +526,18 @@ PyImaging_TgaRleEncoderNew(PyObject* self, PyObject* args)
     char *rawmode;
     Py_ssize_t ystep = 1;
 
-    if (!PyArg_ParseTuple(args, "ss|n", &mode, &rawmode, &ystep))
+    if (!PyArg_ParseTuple(args, "ss|n", &mode, &rawmode, &ystep)) {
         return NULL;
+    }
 
     encoder = PyImaging_EncoderNew(0);
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
-    if (get_packer(encoder, mode, rawmode) < 0)
+    if (get_packer(encoder, mode, rawmode) < 0) {
         return NULL;
+    }
 
     encoder->encode = ImagingTgaRleEncode;
 
@@ -535,11 +558,13 @@ PyImaging_XbmEncoderNew(PyObject* self, PyObject* args)
     ImagingEncoderObject* encoder;
 
     encoder = PyImaging_EncoderNew(0);
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
-    if (get_packer(encoder, "1", "1;R") < 0)
+    if (get_packer(encoder, "1", "1;R") < 0) {
         return NULL;
+    }
 
     encoder->encode = ImagingXbmEncode;
 
@@ -567,22 +592,25 @@ PyImaging_ZipEncoderNew(PyObject* self, PyObject* args)
     Py_ssize_t compress_type = -1;
     char* dictionary = NULL;
     Py_ssize_t dictionary_size = 0;
-    if (!PyArg_ParseTuple(args, "ss|nnn"PY_ARG_BYTES_LENGTH, &mode, &rawmode,
+    if (!PyArg_ParseTuple(args, "ss|nnny#", &mode, &rawmode,
                           &optimize,
                           &compress_level, &compress_type,
-                          &dictionary, &dictionary_size))
+                          &dictionary, &dictionary_size)) {
         return NULL;
+    }
 
     /* Copy to avoid referencing Python's memory */
     if (dictionary && dictionary_size > 0) {
         /* malloc check ok, size comes from PyArg_ParseTuple */
         char* p = malloc(dictionary_size);
-        if (!p)
+        if (!p) {
             return PyErr_NoMemory();
+        }
         memcpy(p, dictionary, dictionary_size);
         dictionary = p;
-    } else
+    } else {
         dictionary = NULL;
+    }
 
     encoder = PyImaging_EncoderNew(sizeof(ZIPSTATE));
     if (encoder == NULL) {
@@ -598,9 +626,10 @@ PyImaging_ZipEncoderNew(PyObject* self, PyObject* args)
     encoder->encode = ImagingZipEncode;
     encoder->cleanup = ImagingZipEncodeCleanup;
 
-    if (rawmode[0] == 'P')
+    if (rawmode[0] == 'P') {
         /* disable filtering */
         ((ZIPSTATE*)encoder->state.context)->mode = ZIP_PNG_PALETTE;
+    }
 
     ((ZIPSTATE*)encoder->state.context)->optimize = optimize;
     ((ZIPSTATE*)encoder->state.context)->compress_level = compress_level;
@@ -642,7 +671,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
     // This list also exists in TiffTags.py
     const int core_tags[] = {
         256, 257, 258, 259, 262, 263, 266, 269, 274, 277, 278, 280, 281, 340,
-        341, 282, 283, 284, 286, 287, 296, 297, 321, 338, 32995, 32998, 32996,
+        341, 282, 283, 284, 286, 287, 296, 297, 320, 321, 338, 32995, 32998, 32996,
         339, 32997, 330, 531, 530, 65537
     };
 
@@ -676,11 +705,13 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
     TRACE(("new tiff encoder %s fp: %d, filename: %s \n", compname, fp, filename));
 
     encoder = PyImaging_EncoderNew(sizeof(TIFFSTATE));
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
-    if (get_packer(encoder, mode, rawmode) < 0)
+    if (get_packer(encoder, mode, rawmode) < 0) {
         return NULL;
+    }
 
     if (! ImagingLibTiffEncodeInit(&encoder->state, filename, fp)) {
         Py_DECREF(encoder);
@@ -693,7 +724,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
         item = PyList_GetItem(tags, pos);
         // We already checked that tags is a 2-tuple list.
         key = PyTuple_GetItem(item, 0);
-        key_int = (int)PyInt_AsLong(key);
+        key_int = (int)PyLong_AsLong(key);
         value = PyTuple_GetItem(item, 1);
         status = 0;
         is_core_tag = 0;
@@ -710,7 +741,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
         if (!is_core_tag) {
             PyObject *tag_type = PyDict_GetItem(types, key);
             if (tag_type) {
-                int type_int = PyInt_AsLong(tag_type);
+                int type_int = PyLong_AsLong(tag_type);
                 if (type_int >= TIFF_BYTE && type_int <= TIFF_DOUBLE) {
                     type = (TIFFDataType)type_int;
                 }
@@ -721,7 +752,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
         if (type == TIFF_NOTYPE) {
             // Autodetect type. Types should not be changed for backwards
             // compatibility.
-            if (PyInt_Check(value)) {
+            if (PyLong_Check(value)) {
                 type = TIFF_LONG;
             } else if (PyFloat_Check(value)) {
                 type = TIFF_DOUBLE;
@@ -730,12 +761,6 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
             }
         }
 
-        if (PyBytes_Check(value) &&
-                (type == TIFF_BYTE || type == TIFF_UNDEFINED)) {
-            // For backwards compatibility
-            type = TIFF_ASCII;
-        }
-
         if (PyTuple_Check(value)) {
             Py_ssize_t len;
             len = PyTuple_Size(value);
@@ -749,7 +774,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
             if (type == TIFF_NOTYPE) {
                 // Autodetect type based on first item. Types should not be
                 // changed for backwards compatibility.
-                if (PyInt_Check(PyTuple_GetItem(value,0))) {
+                if (PyLong_Check(PyTuple_GetItem(value,0))) {
                     type = TIFF_LONG;
                 } else if (PyFloat_Check(PyTuple_GetItem(value,0))) {
                     type = TIFF_FLOAT;
@@ -759,25 +784,40 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
 
         if (!is_core_tag) {
             // Register field for non core tags.
+            if (type == TIFF_BYTE) {
+                is_var_length = 1;
+            }
             if (ImagingLibTiffMergeFieldInfo(&encoder->state, type, key_int, is_var_length)) {
                 continue;
             }
         }
 
-        if (is_var_length) {
+        if (type == TIFF_BYTE || type == TIFF_UNDEFINED) {
+            status = ImagingLibTiffSetField(&encoder->state,
+                    (ttag_t) key_int,
+                    PyBytes_Size(value), PyBytes_AsString(value));
+        } else if (is_var_length) {
             Py_ssize_t len,i;
             TRACE(("Setting from Tuple: %d \n", key_int));
             len = PyTuple_Size(value);
 
-            if (type == TIFF_BYTE) {
-                UINT8 *av;
+            if (key_int == TIFFTAG_COLORMAP) {
+                int stride = 256;
+                if (len != 768) {
+                    PyErr_SetString(PyExc_ValueError, "Requiring 768 items for for Colormap");
+                    return NULL;
+                }
+                UINT16 *av;
                 /* malloc check ok, calloc checks for overflow */
-                av = calloc(len, sizeof(UINT8));
+                av = calloc(len, sizeof(UINT16));
                 if (av) {
                     for (i=0;i<len;i++) {
-                        av[i] = (UINT8)PyInt_AsLong(PyTuple_GetItem(value,i));
+                        av[i] = (UINT16)PyLong_AsLong(PyTuple_GetItem(value,i));
                     }
-                    status = ImagingLibTiffSetField(&encoder->state, (ttag_t) key_int, len, av);
+                    status = ImagingLibTiffSetField(&encoder->state, (ttag_t) key_int,
+                                                    av,
+                                                    av + stride,
+                                                    av + stride * 2);
                     free(av);
                 }
             } else if (type == TIFF_SHORT) {
@@ -786,7 +826,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
                 av = calloc(len, sizeof(UINT16));
                 if (av) {
                     for (i=0;i<len;i++) {
-                        av[i] = (UINT16)PyInt_AsLong(PyTuple_GetItem(value,i));
+                        av[i] = (UINT16)PyLong_AsLong(PyTuple_GetItem(value,i));
                     }
                     status = ImagingLibTiffSetField(&encoder->state, (ttag_t) key_int, len, av);
                     free(av);
@@ -797,7 +837,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
                 av = calloc(len, sizeof(UINT32));
                 if (av) {
                     for (i=0;i<len;i++) {
-                        av[i] = (UINT32)PyInt_AsLong(PyTuple_GetItem(value,i));
+                        av[i] = (UINT32)PyLong_AsLong(PyTuple_GetItem(value,i));
                     }
                     status = ImagingLibTiffSetField(&encoder->state, (ttag_t) key_int, len, av);
                     free(av);
@@ -808,7 +848,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
                 av = calloc(len, sizeof(INT8));
                 if (av) {
                     for (i=0;i<len;i++) {
-                        av[i] = (INT8)PyInt_AsLong(PyTuple_GetItem(value,i));
+                        av[i] = (INT8)PyLong_AsLong(PyTuple_GetItem(value,i));
                     }
                     status = ImagingLibTiffSetField(&encoder->state, (ttag_t) key_int, len, av);
                     free(av);
@@ -819,7 +859,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
                 av = calloc(len, sizeof(INT16));
                 if (av) {
                     for (i=0;i<len;i++) {
-                        av[i] = (INT16)PyInt_AsLong(PyTuple_GetItem(value,i));
+                        av[i] = (INT16)PyLong_AsLong(PyTuple_GetItem(value,i));
                     }
                     status = ImagingLibTiffSetField(&encoder->state, (ttag_t) key_int, len, av);
                     free(av);
@@ -830,7 +870,7 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
                 av = calloc(len, sizeof(INT32));
                 if (av) {
                     for (i=0;i<len;i++) {
-                        av[i] = (INT32)PyInt_AsLong(PyTuple_GetItem(value,i));
+                        av[i] = (INT32)PyLong_AsLong(PyTuple_GetItem(value,i));
                     }
                     status = ImagingLibTiffSetField(&encoder->state, (ttag_t) key_int, len, av);
                     free(av);
@@ -862,19 +902,19 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
             if (type == TIFF_SHORT) {
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
-                        (UINT16)PyInt_AsLong(value));
+                        (UINT16)PyLong_AsLong(value));
             } else if (type == TIFF_LONG) {
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
-                        (UINT32)PyInt_AsLong(value));
+                        (UINT32)PyLong_AsLong(value));
             } else if (type == TIFF_SSHORT) {
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
-                        (INT16)PyInt_AsLong(value));
+                        (INT16)PyLong_AsLong(value));
             } else if (type == TIFF_SLONG) {
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
-                        (INT32)PyInt_AsLong(value));
+                        (INT32)PyLong_AsLong(value));
             } else if (type == TIFF_FLOAT) {
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
@@ -883,14 +923,10 @@ PyImaging_LibTiffEncoderNew(PyObject* self, PyObject* args)
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
                         (FLOAT64)PyFloat_AsDouble(value));
-            } else if (type == TIFF_BYTE) {
-                status = ImagingLibTiffSetField(&encoder->state,
-                        (ttag_t) key_int,
-                        (UINT8)PyInt_AsLong(value));
             } else if (type == TIFF_SBYTE) {
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
-                        (INT8)PyInt_AsLong(value));
+                        (INT8)PyLong_AsLong(value));
             } else if (type == TIFF_ASCII) {
                 status = ImagingLibTiffSetField(&encoder->state,
                         (ttag_t) key_int,
@@ -984,7 +1020,7 @@ static unsigned int* get_qtables_arrays(PyObject* qtables, int* qtablesLen) {
         }
         table_data = PySequence_Fast(table, "expected a sequence");
         for (j = 0; j < DCTSIZE2; j++) {
-            qarrays[i * DCTSIZE2 + j] = PyInt_AS_LONG(PySequence_Fast_GET_ITEM(table_data, j));
+            qarrays[i * DCTSIZE2 + j] = PyLong_AS_LONG(PySequence_Fast_GET_ITEM(table_data, j));
         }
         Py_DECREF(table_data);
     }
@@ -1024,16 +1060,18 @@ PyImaging_JpegEncoderNew(PyObject* self, PyObject* args)
     char* rawExif = NULL;
     Py_ssize_t rawExifLen = 0;
 
-    if (!PyArg_ParseTuple(args, "ss|nnnnnnnnO"PY_ARG_BYTES_LENGTH""PY_ARG_BYTES_LENGTH,
+    if (!PyArg_ParseTuple(args, "ss|nnnnnnnnOy#y#",
                           &mode, &rawmode, &quality,
                           &progressive, &smooth, &optimize, &streamtype,
                           &xdpi, &ydpi, &subsampling, &qtables, &extra, &extra_size,
-                          &rawExif, &rawExifLen))
+                          &rawExif, &rawExifLen)) {
         return NULL;
+    }
 
     encoder = PyImaging_EncoderNew(sizeof(JPEGENCODERSTATE));
-    if (encoder == NULL)
+    if (encoder == NULL) {
         return NULL;
+    }
 
     // libjpeg-turbo supports different output formats.
     // We are choosing Pillow's native format (3 color bytes + 1 padding)
@@ -1042,8 +1080,9 @@ PyImaging_JpegEncoderNew(PyObject* self, PyObject* args)
         rawmode = "RGBX";
     }
 
-    if (get_packer(encoder, mode, rawmode) < 0)
+    if (get_packer(encoder, mode, rawmode) < 0) {
         return NULL;
+    }
 
     // Freed in JpegEncode, Case 5
     qarrays = get_qtables_arrays(qtables, &qtablesLen);
@@ -1051,24 +1090,29 @@ PyImaging_JpegEncoderNew(PyObject* self, PyObject* args)
     if (extra && extra_size > 0) {
         /* malloc check ok, length is from python parsearg */
         char* p = malloc(extra_size); // Freed in JpegEncode, Case 5
-        if (!p)
+        if (!p) {
             return PyErr_NoMemory();
+        }
         memcpy(p, extra, extra_size);
         extra = p;
-    } else
+    } else {
         extra = NULL;
+    }
 
     if (rawExif && rawExifLen > 0) {
         /* malloc check ok, length is from python parsearg */
         char* pp = malloc(rawExifLen); // Freed in JpegEncode, Case 5
         if (!pp) {
-            if (extra) free(extra);
+            if (extra) {
+                free(extra);
+            }
             return PyErr_NoMemory();
         }
         memcpy(pp, rawExif, rawExifLen);
         rawExif = pp;
-    } else
+    } else {
         rawExif = NULL;
+    }
 
     encoder->encode = ImagingJpegEncode;
 
@@ -1096,7 +1140,7 @@ PyImaging_JpegEncoderNew(PyObject* self, PyObject* args)
 
 
 /* -------------------------------------------------------------------- */
-/* JPEG	2000								*/
+/* JPEG 2000                                                            */
 /* -------------------------------------------------------------------- */
 
 #ifdef HAVE_OPENJPEG
@@ -1109,13 +1153,15 @@ j2k_decode_coord_tuple(PyObject *tuple, int *x, int *y)
     *x = *y = 0;
 
     if (tuple && PyTuple_Check(tuple) && PyTuple_GET_SIZE(tuple) == 2) {
-        *x = (int)PyInt_AsLong(PyTuple_GET_ITEM(tuple, 0));
-        *y = (int)PyInt_AsLong(PyTuple_GET_ITEM(tuple, 1));
+        *x = (int)PyLong_AsLong(PyTuple_GET_ITEM(tuple, 0));
+        *y = (int)PyLong_AsLong(PyTuple_GET_ITEM(tuple, 1));
 
-        if (*x < 0)
+        if (*x < 0) {
             *x = 0;
-        if (*y < 0)
+        }
+        if (*y < 0) {
             *y = 0;
+        }
     }
 }
 
@@ -1145,45 +1191,50 @@ PyImaging_Jpeg2KEncoderNew(PyObject *self, PyObject *args)
                           &quality_mode, &quality_layers, &num_resolutions,
                           &cblk_size, &precinct_size,
                           &irreversible, &progression, &cinema_mode,
-                          &fd))
+                          &fd)) {
         return NULL;
+    }
 
-    if (strcmp (format, "j2k") == 0)
+    if (strcmp (format, "j2k") == 0) {
         codec_format = OPJ_CODEC_J2K;
-    else if (strcmp (format, "jpt") == 0)
+    } else if (strcmp (format, "jpt") == 0) {
         codec_format = OPJ_CODEC_JPT;
-    else if (strcmp (format, "jp2") == 0)
+    } else if (strcmp (format, "jp2") == 0) {
         codec_format = OPJ_CODEC_JP2;
-    else
+    } else {
         return NULL;
+    }
 
-    if (strcmp(progression, "LRCP") == 0)
+    if (strcmp(progression, "LRCP") == 0) {
         prog_order = OPJ_LRCP;
-    else if (strcmp(progression, "RLCP") == 0)
+    } else if (strcmp(progression, "RLCP") == 0) {
         prog_order = OPJ_RLCP;
-    else if (strcmp(progression, "RPCL") == 0)
+    } else if (strcmp(progression, "RPCL") == 0) {
         prog_order = OPJ_RPCL;
-    else if (strcmp(progression, "PCRL") == 0)
+    } else if (strcmp(progression, "PCRL") == 0) {
         prog_order = OPJ_PCRL;
-    else if (strcmp(progression, "CPRL") == 0)
+    } else if (strcmp(progression, "CPRL") == 0) {
         prog_order = OPJ_CPRL;
-    else
+    } else {
         return NULL;
+    }
 
-    if (strcmp(cinema_mode, "no") == 0)
+    if (strcmp(cinema_mode, "no") == 0) {
         cine_mode = OPJ_OFF;
-    else if (strcmp(cinema_mode, "cinema2k-24") == 0)
+    } else if (strcmp(cinema_mode, "cinema2k-24") == 0) {
         cine_mode = OPJ_CINEMA2K_24;
-    else if (strcmp(cinema_mode, "cinema2k-48") == 0)
+    } else if (strcmp(cinema_mode, "cinema2k-48") == 0) {
         cine_mode = OPJ_CINEMA2K_48;
-    else if (strcmp(cinema_mode, "cinema4k-24") == 0)
+    } else if (strcmp(cinema_mode, "cinema4k-24") == 0) {
         cine_mode = OPJ_CINEMA4K_24;
-    else
+    } else {
         return NULL;
+    }
 
     encoder = PyImaging_EncoderNew(sizeof(JPEG2KENCODESTATE));
-    if (!encoder)
+    if (!encoder) {
         return NULL;
+    }
 
     encoder->encode = ImagingJpeg2KEncode;
     encoder->cleanup = ImagingJpeg2KEncodeCleanup;
@@ -1211,6 +1262,8 @@ PyImaging_Jpeg2KEncoderNew(PyObject *self, PyObject *args)
             PyErr_SetString(PyExc_ValueError,
                             "JPEG 2000 tile offset too small; top left tile must "
                             "intersect image area");
+            Py_DECREF(encoder);
+            return NULL;
         }
 
         if (context->tile_offset_x > context->offset_x
